function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import { createCache } from './cache';
import mitt from 'mitt';
import fetch from 'isomorphic-unfetch';
import { registerClientForLocalMutations } from './mutate-browser-cache';
export var defaultDispatcher = function defaultDispatcher(action) {
  return fetch(action.url, action).then(function (resp) {
    return resp.json().then(function (data) {
      return {
        data: data,
        status: resp.status,
        headers: _extends({}, resp.headers)
      };
    })["catch"](function (error) {
      return {
        error: error,
        status: resp.status,
        headers: _extends({}, resp.headers)
      };
    });
  })["catch"](function (error) {
    return {
      error: error,
      status: error.status,
      headers: error.headers
    };
  });
};

var defaultIdFromAction = function defaultIdFromAction(action) {
  return action.url;
};

var defaultShouldCacheAction = function defaultShouldCacheAction(action) {
  return action.method === 'GET';
};

export var FetchClient =
/*#__PURE__*/
function () {
  /**
   * A function that creates an id from an action
   *
   * @type {IdFromActionFn}
   * @memberof FetchClient
   */

  /**
   * A function that tells the client if it should cache the action result
   *
   * @type {ShouldCacheFetchResultFn}
   * @memberof FetchClient
   */

  /**
   * An object that matches the action id to its promise on queue
   *
   * @protected
   * @type {{ [fetchId in string]: Promise<any> }}
   * @memberof FetchClient
   */

  /**
   * Mitt instance that handles all events
   * @protected
   * @type {mitt.Emitter}
   * @memberof FetchClient
   */
  function FetchClient(cache, dispatcher, idFromAction, shouldCacheAction) {
    this.cache = cache;
    this.dispatcher = dispatcher;
    this.idFromAction = idFromAction || defaultIdFromAction;
    this.shouldCacheFetchResult = shouldCacheAction || defaultShouldCacheAction;
    this.queueMap = {};
    this.emitter = mitt();
  }
  /**
   * Dispatchs an action and adds it to queue. If an equal action, compared by id, is found
   * we reuse the same promise and return it. When resolved we emit to all listeners that
   * the value has changed.
   * @param action Action to be dispatch
   */


  var _proto = FetchClient.prototype;

  _proto.dispatch = function dispatch(action, givenCacheId) {
    var _this = this;

    var fetchId = givenCacheId || this.idFromAction(action); // Return promise in queue if it exists

    if (this.queueMap[fetchId]) {
      return this.queueMap[fetchId];
    }

    var promise = this.dispatcher(action);
    this.queueMap[fetchId] = promise;
    return promise.then(function (value) {
      // Remove this action from running actions
      delete _this.queueMap[fetchId];

      var shouldCache = _this.shouldCacheFetchResult(action);

      if (shouldCache) {
        _this.cache.set(fetchId, value);
      }

      _this.emitter.emit('update', {
        fetchId: fetchId,
        data: value
      });

      return value;
    });
  }
  /**
   *
   * @param fetchId
   * @param handlersMap Map of event handle
   * @returns A function that removes the listener
   */
  ;

  _proto.listen = function listen(fetchId, handlersMap) {
    var _this2 = this;

    var handler = function handler(eventName, event) {
      if (eventName && event.fetchId === fetchId && handlersMap[eventName]) {
        handlersMap[eventName](event.data);
      }
    };

    this.emitter.on('*', handler);
    return function () {
      return _this2.emitter.off('*', handler);
    };
  }
  /**
   * Get an item from cache by it's fetch id
   * @param fetchId
   */
  ;

  _proto.get = function get(fetchId) {
    var _this$cache$get;

    return (_this$cache$get = this.cache.get(fetchId)) === null || _this$cache$get === void 0 ? void 0 : _this$cache$get.value;
  }
  /**
   * Create or update a value on cache
   * @param fetchId
   * @param newResult New value to be set to `data` property
   */
  ;

  _proto.set = function set(fetchId, newValueOrUpdateResultFn) {
    var newValue = newValueOrUpdateResultFn;

    if (typeof newValueOrUpdateResultFn === 'function') {
      var _prevValue = this.cache.get(fetchId);

      newValue = newValueOrUpdateResultFn(_prevValue);
    }

    this.cache.set(fetchId, newValue);
    this.emitter.emit('update', {
      fetchId: fetchId,
      data: newValueOrUpdateResultFn
    });
  };

  /**
   * Waits until the running fetchs are resolved
   */
  _proto.waitForFetchs = function waitForFetchs() {
    return Promise.all(this.queue).then(function () {
      return void 0;
    });
  };

  _createClass(FetchClient, [{
    key: "queue",
    get: function get() {
      return Object.values(this.queueMap);
    }
  }]);

  return FetchClient;
}();
export var createClient = function createClient(_ref) {
  var cache = _ref.cache,
      initialCacheState = _ref.initialCacheState,
      _ref$dispatcher = _ref.dispatcher,
      dispatcher = _ref$dispatcher === void 0 ? defaultDispatcher : _ref$dispatcher;
  var client = new FetchClient(cache || createCache({
    initialState: initialCacheState
  }), dispatcher);

  if (typeof window !== 'undefined') {
    registerClientForLocalMutations(client);
  }

  return client;
};